#!/usr/bin/env perl -w
# @(#) XML3gppParser.pl 1.6.0 05/07/25 13:24:53
# @(#) 

# XML3gppParser.pl
# Convert the XML data from Lucent UTRAN (theoretically, from any 3GPP system)
# to a delimited ASCII format that the loader can read.
# The output of this script is piped into the preprocessor
#
# This script will either:
#  Convert the specified file in the specified directory
#  Convert all files in the specified directory that match NODEBXMLPATTERN
# See also the usage() subroutine at the end.
#
# The output of this script is piped into the preprocessor. If the output
# contains error it will be very hard to remove the errorious data from the 
# preprocessor.
# The parser does not know if the XML is valid until it has parsed the whole
# doucment.
# Therefore the parser collects the output into an arry and then prints the
# array once it knows parsing is successful.
#
# As the NodeB and RNC files are small individually this buffering should cause
# minimal performance impact.
# (The performance issues is the large number of nodeb files)

use strict;

use lib $ENV{PM_HOME}."/sw/perl/lib";
use lib $ENV{LuUTRAN_HOME}."/vendor/Lucent/UTRAN/1.3/LuUTRAN_loader/Common/scripts";

use XML::Parser;
use File::Basename;
use modules::config_functions;
use modules::time_functions;

use IO::File;

# Initialise PPL
sub initialiseConfiguration {
  my %user_defaults=
  (
    # Time stamp conversion not used in the XML parser
    # Dummy variables need for module
    INPUT_TSTAMP_FORMAT  => "",
    OUTPUT_TSTAMP_FORMAT => "",

    LOGFILENAME => "xmlParserLog" . todays_date(),
  );

  register_config_defaults (%user_defaults);

  register_read_only_variables
    ("LOGFILENAME");

  register_mandatory_variables
    ("LOGFILEDIR",
    );
}

# Global variables used in main routing and in parsing XML
my $config= {};
my $outsep;
my $xml_file;

# Global variables used to parse the XML file
my ($success, $output_sn, $output_moid);
my @output;
my $output_data;
my $suspect_flag=0;

my $in_tag;

my ($label, $sn, $moid, $per_len);

my @counter_names;
my $current_counter;
my $value;
my @values;

my $first;

### D20060413 : SEAde60392
my $str_collection;

# Handler procedures used to parse the XML file
sub handle_init {
  my ($parser) = @_;

  # Reset the variables from last file parsed
  $success = 1;
#  @output = (  "FILENAME" . $outsep . $xml_file . "\n"  );
  print  "FILENAME" . $outsep . $xml_file . "\n";
  $first = 1;
  $sn = undef;
  $output_sn = 0;
  $output_data = 0;
  $label="";
}

sub handle_start {
  my ($parser, $element, @attributes_values) = @_;

  return unless $success;

  $in_tag = $element;
  $str_collection = '';

  # Reset variables used for each block
  # Tests for tag names are in order of frequency -
  # <r> occurs much more often than <mv>, which occurs
  # much more often than <mi>.
  if ($element eq "r") {
    $value= "";
  } elsif ($element eq "mv") {
    @values = ();
    $current_counter=0;
    $moid="";
    $output_moid=0;
    $suspect_flag=0;
  } elsif ($element eq "mi") {
    @counter_names = ();
    $per_len = undef;
  } 
}

sub handle_end {
  my ($parser, $element) = @_;
  
  return unless $success;

  ### start of D20060413: SEAde60392
  # Tests are in approximate order of tag frequency.
  if ($in_tag eq "r") {
    $value= $str_collection;
  } elsif ($in_tag eq "moid") {
    $moid = $str_collection;
    if (defined $per_len) {
      push @values, ( join ($outsep,
                            "MOID",
                            $moid,
                            $per_len) . "\n");
      $output_moid = 1;
    } else {
      messageHandler($config, 1, "$xml_file : Missing gp tag in XML for group containing moid : $moid");
      $success=0;
    }
  } elsif ($in_tag eq "sf") {
    # Set the suspect flag if the data is explicity marked suspect
    if (lc $str_collection eq "true") {
      $suspect_flag = 1;
      messageHandler($config, 1, "$xml_file : Data marked as suspect and will not be loaded for moid : $moid");
    }
  } elsif ($in_tag eq "mt") {
    # Collect the counter names onto an array
    # This array is used to determine the counter name when processing the values
    push @counter_names ,$str_collection;
  } elsif ($in_tag eq "sn") {
    $sn = $str_collection;
  } elsif ($in_tag eq "cbt") {
    if (defined $sn) {
      print  join ($outsep,
		   "SN",
		   $sn,
		   $str_collection) . "\n";
      $output_sn = 1;
    }
  } elsif ($in_tag eq "neun") {
    $label=$str_collection;
  } elsif ($in_tag eq "gp") {
    $per_len = $str_collection;
  }

  ### end of D20060413: SEAde60392

  $in_tag = '';

  if ($element eq "r") {
    # Collect the data values when the end r tag is seen to ensure
    # collection of null values 
    $current_counter+=1;
    if ($current_counter > scalar @counter_names) {
      messageHandler($config, 1, "$xml_file : More values than counter names for moid : $moid");
      $success = 0;
    } elsif ( ! $output_moid ) {
      messageHandler($config, 1, "$xml_file : Missing moid tag in XML");
      $success = 0;
    } else {
      my $counter = $counter_names[$current_counter-1];
      # Data values are initially put on the values array
      # so they can be discarded if the suspect flag is true
      push @values, ($counter . $outsep . $value . "\n");
    }
  } elsif ($element eq "mv") {
    if ($current_counter < scalar @counter_names) {
      messageHandler($config, 1, "$xml_file : Missing values in XML for moid : $moid");
      $success = 0;
    # If the suspect flag is not set
    #   Place the data into the output array
    } elsif (not $suspect_flag) {
      print @values;
      $output_data = 1;
    }
  } elsif ($first and $element eq "neun") {
    print join ($outsep,
		"LABEL",
		$label) . "\n";
    $first = 0;
  }
}

sub handle_char {
  my ($parser, $string) = @_;
  
  return unless $success;

  ### D20060413: SEAde60392
  ### for unknown reason, the XML::Parser split the string twice occasionally.
  ### To address this weird behaviour, we collect all string between handle_char first.
  ### and build up the array in handle_end later.

  if ($in_tag)
  {
     chomp ($string);
     $str_collection = $str_collection . $string;
  }

}


sub handle_final {
  my ($parser) = @_;
  
  if ($success) {
    if (! $output_sn) {
      messageHandler($config, 1, "$xml_file : Missing sn tag in XML");
      $success=0;
    } elsif ( ! $output_data ) {
      messageHandler($config, 1, "$xml_file : No data to output");
      $success=0;
    } else {
#      for (@output) {
#        print;
#      }
    }
  }

  return $success;
}


# Start of main action
my $SUCCESS = 1;
my $DATA = 0;

my $start_time= time;
my $scriptname= basename $0;

my ($config_file, $in_dir, $file_name)= @ARGV[0..2];
usage() unless
  defined $config_file and
  defined $in_dir;

initialiseConfiguration();
readConfiguration($config_file, $config);

$outsep = $config->{OUTSEP};

messageHandler ($config, 2, "Processing started at ", scalar localtime);

my @xml_files;

# If file name specified:
#   Use file name
# Else
#   Find all files in specified dir that match the NODEBXMLPATTERN
if (defined $file_name) {
  push @xml_files, $file_name; 
} else {
  unless (opendir DIR, $in_dir) {
    messageHandler($config, 0, "Couldn't open directory $in_dir for reading: $!");
    exit 1;
  }
  my $pattern = $config->{NODEBXMLPATTERN};
  @xml_files = grep $_ =~ /$pattern/o && $_ ne "." && $_ ne "..", readdir DIR;
  close DIR;
}

# Initialise the parser specifing the handlers
my $parser = new XML::Parser(Handlers => {Init  => \&handle_init,
                                          Start => \&handle_start,
                                          End   => \&handle_end,
                                          Char  => \&handle_char,
                                          Final => \&handle_final});

# Parse each XML file in turn
for my $file (@xml_files) {
  messageHandler ($config, 2, "Processing file $file");
  # Assume that converting this file failed unless we know it succeeded.

  # Utilising a scalar for the file handle means we can persaude
  # XML PARSER to unzip the file.
  my $handle = new IO::File;

  if (open($handle, adjust_open_string_for_gzip($in_dir . "/" . $file))) {

    # Copy $file to $xml_file as $file is not available in the Handler routines
    $xml_file=$file;

    my $result = eval { $parser->parse($handle) };

    if ($@) {
      messageHandler ($config, 1, "Processing of $xml_file failed:");
      messageHandler ($config, 1, "$@");
      $SUCCESS=0;
    } elsif ($result) {
      $DATA=1;
    } else {
      $SUCCESS=0;
    }

    close($handle);

  } else {
    messageHandler ($config, 1, "Processing of $file failed:\n");
    messageHandler ($config, 1, "Cannot open file : $!");
  }

} # foreach $xml_file

messageHandler ($config, 2, "processing finished at ", scalar localtime);
messageHandler ($config, 2, "(Elapsed time was ", time - $start_time, " seconds.)");

if ($SUCCESS and $DATA) {
  open(SUCCESS , ">/tmp/Lucent_UTRAN_XML3gppParser." . getppid);
  close(SUCCESS);
  exit 0;
} else {
  exit 1;
}


sub usage {
  print STDERR <<EOD;

Usage: $scriptname <config file> <in directory> [<file name>] 

where:
<config file> is a file that controls the conversion process.
<in directory> is the directory containing the XML files.
Output is to standard output.

If a <file name> is specifed ths program converts the XML to an
intermediate ASCII format and sends the infromation to standard out.

If a <file name> is not specified all files that match the NODEBXMLPATTERN 
in the <config file> are processed. The assumption is that NodeB XML files
within the directory are for a given RNC for a given timestamp.

EOD

exit 1;

} # sub usage

